import json


class SubstackAPIException(Exception):
    def __init__(self, status_code, text):
        try:
            json_res = json.loads(text)
        except ValueError:
            self.message = f"Invalid JSON error message from Substack: {text}"
        else:
            self.message = ", ".join(
                list(
                    map(lambda error: error.get("msg", ""), json_res.get("errors", []))
                )
            )
            self.message = self.message or json_res.get("error", "")
        self.status_code = status_code

    def __str__(self):
        return f"APIError(code={self.status_code}): {self.message}"


class SubstackRequestException(Exception):
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return f"SubstackRequestException: {self.message}"



import logging

import requests

from substack.exceptions import SubstackAPIException, SubstackRequestException

logger = logging.getLogger(__name__)


class Api:
	"""

	A python interface into the Substack API

	"""

	def __init__(
			self,
			email = None,
			password = None,
			base_url = None,
			publication_url = None,
			debug = False,
	):
		"""

		To create an instance of the substack.Api class:
			>>> import substack
			>>> api = substack.Api(email="substack email", password="substack password")

		Args:
		  email:
		  password:
		  base_url:
			The base URL to use to contact the Substack API.
			Defaults to https://substack.com/api/v1.
		"""
		self.base_url = base_url or "https://substack.com/api/v1"
		self.publication_url = f"{publication_url}/api/v1"

		if debug:
			logging.basicConfig()
			logging.getLogger().setLevel(logging.DEBUG)

		self._session = requests.Session()

		if email is not None and password is not None:
			self.login(email, password)

	def login(self, email, password) -> dict:
		"""

		Login to the substack account.

		Args:
		  email: substack account email
		  password: substack account password
		"""

		response = self._session.post(
			f"{self.base_url}/login",
			json={
				"captcha_response": None,
				"email": email,
				"for_pub": "",
				"password": password,
				"redirect": "/",
			},
		)
		return Api._handle_response(response=response)

	@staticmethod
	def _handle_response(response: requests.Response):
		"""

		Internal helper for handling API responses from the Substack server.
		Raises the appropriate exceptions when necessary; otherwise, returns the
		response.

		"""

		if not (200 <= response.status_code < 300):
			raise SubstackAPIException(response.status_code, response.text)
		try:
			return response.json()
		except ValueError:
			raise SubstackRequestException("Invalid Response: %s" % response.text)

	def get_publication_users(self):
		"""

		:return:
		"""
		response = self._session.get(f"{self.publication_url}/publication/users")

		return Api._handle_response(response=response)

	def get_posts(self) -> dict:
		"""

		:return:
		"""
		response = self._session.get(f"{self.base_url}/reader/posts")

		return Api._handle_response(response=response)

	def get_drafts(self, filter = None, offset = None, limit = None):
		response = self._session.get(
			f"{self.publication_url}/drafts",
			params={"filter": filter, "offset": offset, "limit": limit},
		)
		return Api._handle_response(response=response)

	def delete_draft(self, draft_id):
		response = self._session.delete(
			f"{self.publication_url}/drafts/{draft_id}"
		)
		return Api._handle_response(response=response)

	def post_draft(self, body) -> dict:
		"""

		Args:
		  body:

		Returns:

		"""
		response = self._session.post(f"{self.publication_url}/drafts", json=body)
		return Api._handle_response(response=response)

	def put_draft(
			self,
			draft,
			title = None,
			subtitle = None,
			body = None,
			cover_image = None,
	) -> dict:
		"""

		Args:
			draft: draft id
			title:
			subtitle:
			body:
			cover_image:

		Returns:

		"""

		response = self._session.put(
			f"{self.publication_url}/drafts/{draft}",
			json={
				"draft_title": title,
				"draft_subtitle": subtitle,
				"draft_body": body,
				"cover_image": cover_image,
			},
		)
		return Api._handle_response(response=response)

	def prepublish_draft(self, draft) -> dict:
		"""

		Args:
			draft: draft id

		Returns:

		"""

		response = self._session.get(
			f"{self.publication_url}/drafts/{draft}/prepublish"
		)
		return Api._handle_response(response=response)

	def publish_draft(
			self, draft, send: bool = True, share_automatically: bool = False
	) -> dict:
		"""

		Args:
			draft: draft id
			send:
			share_automatically:

		Returns:

		"""
		response = self._session.post(
			f"{self.publication_url}/drafts/{draft}/publish",
			json={"send": send, "share_automatically": share_automatically},
		)
		return Api._handle_response(response=response)

	def get_image(self, image):
		response = self._session.post(
			f"{self.publication_url}/image",
			json={
				"image": image
			},
		)
		return Api._handle_response(response=response)

	def get_categories(self):
		"""

		Retrieve list of all available categories.

		Returns:

		"""
		response = self._session.get(f"{self.base_url}/categories")
		return Api._handle_response(response=response)

	def get_category(self, category_id, category_type, page):
		response = self._session.get(f"{self.base_url}/category/public/{category_id}/{category_type}",
		                             params={"page": page})
		return Api._handle_response(response=response)

	def get_single_category(self, category_id, category_type, page = None,
	                        limit = None):
		"""

		Args:
			category_id:
			category_type: paid or all
			page: by default substack retrieves only the first 25 publications in the category. If this is left None,
				  then all pages will be retrieved. The page size is 25 publications.
			limit:
		Returns:

		"""
		if page is not None:
			output = self.get_category(category_id, category_type, page)
		else:
			publications = []
			page = 0
			while True:
				page_output = self.get_category(category_id, category_type, page)
				publications.extend(page_output.get("publications", []))
				if (limit is not None and limit <= len(publications)) or not page_output.get("more", False):
					publications = publications[:limit]
					break
				page += 1
			output = {
				"publications": publications,
				"more": page_output.get("more", False)
			}
		return output

	def delete_all_drafts(self):
		response = None
		while True:
			drafts = self.get_drafts(filter="draft", limit=10, offset=0)
			if len(drafts) == 0:
				break
			for draft in drafts:
				response = self.delete_draft(draft.get("id"))
		return response



import json


class Post:

	def __init__(self, title, subtitle, user_id):
		self.draft_title = title
		self.draft_subtitle = subtitle
		self.draft_body = {"type": "doc", "content": []}
		self.draft_bylines = [{"id": int(user_id), "is_guest": False}]

	def add(self, item):
		self.draft_body["content"] = self.draft_body.get("content", []) + [{"type": item.get("type")}]
		content = item.get("content")
		if item.get("type") == "captionedImage":
			self.captioned_image(item)
		elif item.get("type") == "youtube2":
			self.youtube(item.get("src"))
		else:
			if content is not None:
				self.add_complex_text(content)

		if item.get("type") == "heading":
			self.attrs(item.get("level", 1))

		marks = item.get("marks")
		if marks is not None:
			self.marks(marks)

		return self

	def paragraph(self, content=None):
		item = {"type": "paragraph"}
		if content is not None:
			item["content"] = content
		return self.add(item)

	def heading(self, content=None, level=1):
		item = {"type": "heading"}
		if content is not None:
			item["content"] = content
		item["level"] = level
		return self.add(item)

	def horizontal_rule(self):
		return self.add({"type": "horizontal_rule"})

	def attrs(self, level):
		content_attrs = self.draft_body["content"][-1].get("attrs", {})
		content_attrs.update({"level": level})
		self.draft_body["content"][-1]["attrs"] = content_attrs
		return self

	def captioned_image(self, value):
		content = self.draft_body["content"][-1].get("content", [])
		content += [{"type": "image2", "attrs": {
			"src": value.get("src"),
			"fullscreen": False,
			"imageSize": value.get("size", "normal"),
			"height": 819,
			"width": 1456,
			"resizeWidth": 728,

			"bytes": None, "alt": None, "title": None, "type": None, "href": None, "belowTheFold": False,
			"internalRedirect": None

		}}]
		self.draft_body["content"][-1]["content"] = content

	def text(self, value):
		"""

		Add text to the last paragraph.

		Args:
			value: Text to add to paragraph.

		Returns:

		"""
		content = self.draft_body["content"][-1].get("content", [])
		content += [{"type": "text", "text": value}]
		self.draft_body["content"][-1]["content"] = content
		return self

	def add_complex_text(self, text):
		if isinstance(text, str):
			self.text(text)
		else:
			for chunk in text:
				if chunk:
					self.text(chunk.get("content")).marks(chunk.get("marks"))

	def marks(self, marks):
		content = self.draft_body["content"][-1].get("content", [])[-1]
		content_marks = content.get("marks", [])
		for mark in marks:
			new_mark = {"type": mark.get("type")}
			if mark.get("type") == "link":
				href = mark.get("href")
				new_mark.update({"attrs": {
					"href": href
				}})
			content_marks.append(new_mark)
		content["marks"] = content_marks
		return self

	def remove_last_paragraph(self):
		del self.draft_body.get("content")[-1]

	def get_draft(self):
		out = vars(self)
		out["draft_body"] = json.dumps(out["draft_body"])
		return out

	def subscribe_with_caption(self, value):
		content = self.draft_body["content"][-1].get("content", [])
		content += [{"type": "subscribeWidget",
		             "attrs": {"url": "%%checkout_url%%", "text": "Subscribe"},
		             "content": [
			             {
				             "type": "ctaCaption",
				             "content": [{"type": "text",
				                          "text": f"Thanks for reading {value}! Subscribe for free to receive new posts and support my work."}]
			             }
		             ]}]
		self.draft_body["content"][-1]["content"] = content
		return self

	def youtube(self, value):
		content_attrs = self.draft_body["content"][-1].get("attrs", {})
		content_attrs.update({"videoId": value})
		self.draft_body["content"][-1]["attrs"] = content_attrs
		return self



"""A library that provides a Python interface to the Substack API."""

__author__ = "Paolo Mazza"
__email__ = "mazzapaolo2019@gmail.com"
__license__ = "MIT License"
__version__ = "1.0"
__url__ = "https://github.com/hogier/python-substack"
__download_url__ = "https://pypi.python.org/pypi/python-substack"
__description__ = "A Python wrapper around the Substack API"

from .api import Api



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['substack']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'python-dotenv>=0.21.0,<0.22.0', 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'python-substack',
    'version': '0.1.4',
    'description': 'A Python wrapper around the Substack API.',
    'long_description': '# Python Substack\n\n# Introduction\n\nThis is an unofficial library providing a Python interface for [Substack](https://substack.com/).\nI am in no way affiliated with Substack. It works with\nPython versions from 3.8+.\n\n# Installation\n\nYou can install python-substack using:\n\n    $ pip install python-substack',
    'author': 'Paolo Mazza',
    'author_email': 'mazzapaolo2019@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/mazza8/python-substack',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
